"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var constants_1 = require("./constants");
var fs = (0, tslib_1.__importStar)(require("fs"));
var path = (0, tslib_1.__importStar)(require("path"));
var btc = require('bindings')('btcjs');
/**
 * Process AST before it's really parsed
 * by ASTParser so we can replace import declaration's
 * with it's AST file contents
 */
var ASTPreprocessor = /** @class */ (function () {
    function ASTPreprocessor(ast, options) {
        var _this = this;
        this.ast = ast;
        this.options = options;
        this.aliases = new Map();
        this.result = ast.reduce(function (result, node) { return result.concat(_this.process(node)); }, new Array());
    }
    ASTPreprocessor.prototype.getResult = function () {
        return this.result;
    };
    ASTPreprocessor.prototype.process = function (ast) {
        var _this = this;
        switch (ast.type) {
            case constants_1.Syntax.Namespace: {
                var body = ast.body.reduce(function (result, item) {
                    return result.concat(_this.process(item));
                }, new Array());
                return [(0, tslib_1.__assign)((0, tslib_1.__assign)({}, ast), { body: body })];
            }
            case constants_1.Syntax.ContainerGroup: {
                var body = ast.body.reduce(function (nodes, node) {
                    return nodes.concat(_this.process(node));
                }, new Array());
                return [(0, tslib_1.__assign)((0, tslib_1.__assign)({}, ast), { body: body })];
            }
            case constants_1.Syntax.ContainerDeclaration:
                return [(0, tslib_1.__assign)((0, tslib_1.__assign)({}, ast), { body: ast.body.reduce(function (params, param) { return (params.concat(_this.process(param))); }, new Array()) })];
            case constants_1.Syntax.Template: {
                var args = ast.arguments.reduce(function (newArgs, arg) {
                    return newArgs.concat(_this.process(arg));
                }, new Array());
                return [(0, tslib_1.__assign)((0, tslib_1.__assign)({}, ast), { arguments: args })];
            }
            case constants_1.Syntax.Identifier: {
                var alias = this.aliases.get(ast.value);
                if (alias)
                    return [alias];
                return [ast];
            }
            case constants_1.Syntax.ContainerParam: {
                var paramType = this.process(ast.paramType)[0];
                return [(0, tslib_1.__assign)((0, tslib_1.__assign)({}, ast), { paramType: paramType })];
            }
            case constants_1.Syntax.ImportDeclaration: {
                var modulePath = path.resolve(this.options.directory, ast.path);
                try {
                    fs.accessSync(modulePath, fs.constants.R_OK);
                }
                catch (reason) {
                    throw new Error("Schema file \"".concat(modulePath, "\" not found or accessible"));
                }
                var preprocessor = new ASTPreprocessor(btc.parse(fs.readFileSync(modulePath).toString('utf8')), {
                    directory: path.dirname(modulePath)
                });
                return preprocessor.getResult();
            }
            case constants_1.Syntax.TemplateDeclaration:
                return [(0, tslib_1.__assign)((0, tslib_1.__assign)({}, ast), { body: this.process(ast.body)[0], arguments: ast.arguments.map(function (node) { return _this.process(node)[0]; }) })];
            case constants_1.Syntax.Alias: {
                var result = this.process(ast.value);
                if (result.length > 1)
                    throw new Error("Alias must have a single value");
                else if (result.length > 0)
                    this.aliases.set(ast.name, result[0]);
                else
                    throw new Error("Unexpected value for alias item");
                return [];
            }
        }
        return [ast];
    };
    return ASTPreprocessor;
}());
exports["default"] = ASTPreprocessor;
