import { ICodeGenerator, IFileResult } from "./code-generator";
import { Node, NodeContainerDeclaration, NodeContainerGroup } from "../ast-parser/node";
import CodeStream from './code-stream';
/**
 * Uses AST nodes to generate code. Attention: It is fully responsible of
 * the *child* to repass it's children to the parent so it can be processed,
 * in case it was not processed by the child or needs to be processed by the parent
 * after that
 */
export default abstract class CodeGeneratorChild implements ICodeGenerator {
    parent: ICodeGenerator;
    cs: CodeStream;
    constructor(parent: ICodeGenerator);
    getGenerator<T extends ICodeGenerator = ICodeGenerator>(name: string): T;
    getCurrentNode(): Node | undefined;
    setParentNode(node?: Node): void;
    getModuleResolver(): import("./module-resolver").default;
    setCurrentNode(node?: Node): void;
    getFiles(): IFileResult[];
    getContainers(): import("../ast-parser/constants").Container[];
    getNodeListFromParamExpression(node: Node): Node[];
    processNode(node: Node): string;
    indentCode(...args: string[]): string;
    indentFrom(start: string, getContent: () => string, end: string): string;
    indentWith(getContent: () => string): string;
    getPrefixPackage(): string[];
    getPackageName(): string;
    getInterfaceName(name: string): string;
    getParamDefaultFromType(node: Node): string;
    getParentNode(): Node | undefined;
    translateParamType(node: Node): string;
    increaseDepth(): void;
    decreaseDepth(): void;
    getContainer(name: string): import("../ast-parser/constants").Container;
    addFile(file: Pick<IFileResult, 'contents' | 'path'>): void;
    getCurrentPath(): string[];
    getClassNameFromList(node: NodeContainerDeclaration | NodeContainerGroup): string;
    getVariableName(name: string, variant?: string, includeComment?: boolean): string;
    abstract generate(node: Node): string;
}
