"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var code_stream_1 = (0, tslib_1.__importDefault)(require("./code-stream"));
/**
 * Uses AST nodes to generate code. Attention: It is fully responsible of
 * the *child* to repass it's children to the parent so it can be processed,
 * in case it was not processed by the child or needs to be processed by the parent
 * after that
 */
var CodeGeneratorChild = /** @class */ (function () {
    function CodeGeneratorChild(parent) {
        this.parent = parent;
        this.cs = new code_stream_1["default"](this);
    }
    CodeGeneratorChild.prototype.getGenerator = function (name) {
        return this.parent.getGenerator(name);
    };
    CodeGeneratorChild.prototype.getCurrentNode = function () {
        return this.parent.getCurrentNode();
    };
    CodeGeneratorChild.prototype.setParentNode = function (node) {
        this.parent.setParentNode(node);
    };
    CodeGeneratorChild.prototype.getModuleResolver = function () {
        return this.parent.getModuleResolver();
    };
    CodeGeneratorChild.prototype.setCurrentNode = function (node) {
        this.parent.setCurrentNode(node);
    };
    CodeGeneratorChild.prototype.getFiles = function () {
        return this.parent.getFiles();
    };
    CodeGeneratorChild.prototype.getContainers = function () {
        return this.parent.getContainers();
    };
    CodeGeneratorChild.prototype.getNodeListFromParamExpression = function (node) {
        return this.parent.getNodeListFromParamExpression(node);
    };
    CodeGeneratorChild.prototype.processNode = function (node) {
        return this.parent.processNode(node);
    };
    CodeGeneratorChild.prototype.indentCode = function () {
        var _a;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        return (_a = this.parent).indentCode.apply(_a, (0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(args), false));
    };
    CodeGeneratorChild.prototype.indentFrom = function (start, getContent, end) {
        var contents = '';
        contents += this.indentCode(start);
        contents += this.indentWith(getContent);
        contents += this.indentCode(end);
        return contents;
    };
    CodeGeneratorChild.prototype.indentWith = function (getContent) {
        var contents = '';
        this.increaseDepth();
        contents += getContent();
        this.decreaseDepth();
        return contents;
    };
    CodeGeneratorChild.prototype.getPrefixPackage = function () {
        return this.parent.getPrefixPackage();
    };
    CodeGeneratorChild.prototype.getPackageName = function () {
        return this.parent.getPackageName();
    };
    CodeGeneratorChild.prototype.getInterfaceName = function (name) {
        return this.parent.getInterfaceName(name);
    };
    CodeGeneratorChild.prototype.getParamDefaultFromType = function (node) {
        return this.parent.getParamDefaultFromType(node);
    };
    CodeGeneratorChild.prototype.getParentNode = function () {
        return this.parent.getParentNode();
    };
    CodeGeneratorChild.prototype.translateParamType = function (node) {
        return this.parent.translateParamType(node);
    };
    CodeGeneratorChild.prototype.increaseDepth = function () {
        return this.parent.increaseDepth();
    };
    CodeGeneratorChild.prototype.decreaseDepth = function () {
        return this.parent.decreaseDepth();
    };
    CodeGeneratorChild.prototype.getContainer = function (name) {
        return this.parent.getContainer(name);
    };
    CodeGeneratorChild.prototype.addFile = function (file) {
        this.parent.addFile(file);
    };
    CodeGeneratorChild.prototype.getCurrentPath = function () {
        return this.parent.getCurrentPath();
    };
    CodeGeneratorChild.prototype.getClassNameFromList = function (node) {
        return this.parent.getClassNameFromList(node);
    };
    CodeGeneratorChild.prototype.getVariableName = function (name, variant, includeComment) {
        return this.parent.getVariableName(name, variant, includeComment);
    };
    return CodeGeneratorChild;
}());
exports["default"] = CodeGeneratorChild;
