"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var cyclic_rc_1 = (0, tslib_1.__importDefault)(require("cyclic-rc"));
var constants_1 = require("../ast-parser/constants");
var __1 = require("..");
var crypto_1 = require("crypto");
var container_group_generator_1 = (0, tslib_1.__importDefault)(require("./container-group-generator"));
var module_resolver_1 = (0, tslib_1.__importDefault)(require("./module-resolver"));
var util_class_code_generator_1 = (0, tslib_1.__importDefault)(require("./util-class-code-generator"));
var data_container_code_generator_1 = (0, tslib_1.__importDefault)(require("./data-container-code-generator"));
var code_stream_1 = (0, tslib_1.__importDefault)(require("./code-stream"));
var container_declaration_generator_1 = (0, tslib_1.__importDefault)(require("./container-declaration-generator"));
var vector_code_generator_1 = (0, tslib_1.__importDefault)(require("./template-generators/vector-code-generator"));
var typed_array_code_generator_1 = (0, tslib_1.__importDefault)(require("./template-generators/typed-array-code-generator"));
var optional_code_generator_1 = (0, tslib_1.__importDefault)(require("./template-generators/optional-code-generator"));
var strict_size_code_generator_1 = (0, tslib_1.__importDefault)(require("./template-generators/strict-size-code-generator"));
var imports_declaration_code_generator_1 = (0, tslib_1.__importDefault)(require("./imports-declaration-code-generator"));
var map_code_generator_1 = (0, tslib_1.__importDefault)(require("./template-generators/map-code-generator"));
var CodeGenerator = /** @class */ (function () {
    function CodeGenerator(nodes, options) {
        if (options === void 0) { options = {}; }
        this.nodes = nodes;
        this.generators = this.getDefaultGenerators();
        this.files = new Array();
        this.path = new Array();
        this.depth = 0;
        this.prefixPackage = new Array();
        this.containers = new Array();
        this.indentationSize = 4;
        if (options.prefixPackage) {
            this.prefixPackage = options.prefixPackage;
        }
        new __1.ASTParser(nodes, (0, tslib_1.__assign)({ namespaceSeparator: '.', 
            /**
             * We can pass empty directory since preprocessor was already
             * executed in these ast nodes
             */
            directory: '', containers: this.containers }, options)).parse();
        if (typeof options.indentationSize !== 'undefined') {
            this.indentationSize = options.indentationSize;
        }
        this.moduleResolver = new module_resolver_1["default"](options.moduleAliases || new Map());
    }
    CodeGenerator.prototype.getModuleResolver = function () {
        return this.moduleResolver;
    };
    CodeGenerator.prototype.getGenerator = function (name) {
        var gen = this.generators.get(name);
        if (!gen) {
            throw new Error("Generator named ".concat(name, " was not found"));
        }
        return gen;
    };
    CodeGenerator.prototype.getFiles = function () {
        return (0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(this.files), false);
    };
    CodeGenerator.prototype.addFile = function (file) {
        this.files.push((0, tslib_1.__assign)((0, tslib_1.__assign)({}, file), { namespace: this.path.slice(0) }));
    };
    CodeGenerator.prototype.getCurrentNode = function () {
        return this.currentNode;
    };
    CodeGenerator.prototype.getPrefixPackage = function () {
        return this.prefixPackage;
    };
    CodeGenerator.prototype.getParentNode = function () {
        return this.parentNode;
    };
    CodeGenerator.prototype.increaseDepth = function () {
        ++this.depth;
    };
    CodeGenerator.prototype.decreaseDepth = function () {
        if (this.depth === 0) {
            throw new Error('Invalid calculation of depth. Depth cannot be lower than 0');
        }
        --this.depth;
    };
    CodeGenerator.prototype.indentCode = function () {
        var e_1, _a;
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var result = '';
        try {
            for (var args_1 = (0, tslib_1.__values)(args), args_1_1 = args_1.next(); !args_1_1.done; args_1_1 = args_1.next()) {
                var arg = args_1_1.value;
                for (var i = 0; i < this.depth; i++) {
                    for (var j = 0; j < this.indentationSize; j++) {
                        result += ' ';
                    }
                }
                result += arg;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (args_1_1 && !args_1_1.done && (_a = args_1["return"])) _a.call(args_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return result;
    };
    CodeGenerator.prototype.getParamDefaultFromType = function (node) {
        if (node.type === constants_1.Syntax.Identifier) {
            if (node.value === constants_1.Generics.UInt32) {
                return '0';
            }
            else if (node.value === constants_1.Generics.Int32 || node.value === constants_1.Generics.Int8 || node.value === constants_1.Generics.UInt8) {
                return '0';
            }
            else if (node.value === constants_1.Generics.Double) {
                return '0';
            }
            else if (node.value === constants_1.Generics.String) {
                return '\'\'';
            }
            return '';
        }
        else if (node.type === constants_1.Syntax.Template) {
            var gen = this.getGenerator("templateGenerator::".concat(node.name));
            return gen.getDefaultValueExpression(node);
        }
        return '';
    };
    CodeGenerator.prototype.getContainers = function () {
        return this.containers;
    };
    CodeGenerator.prototype.setParentNode = function (node) {
        this.parentNode = node;
    };
    CodeGenerator.prototype.processNode = function (item) {
        var _this = this;
        var _a = new code_stream_1["default"](this), write = _a.write, append = _a.append, valueOf = _a.valueOf;
        switch (item.type) {
            case constants_1.Syntax.Namespace: {
                write("export namespace ".concat(item.name, " {\n"), function () {
                    var e_2, _a;
                    _this.setParentNode(item);
                    _this.setCurrentNode(item);
                    _this.path.push(item.name);
                    try {
                        for (var _b = (0, tslib_1.__values)(item.body), _c = _b.next(); !_c.done; _c = _b.next()) {
                            var node = _c.value;
                            append(_this.processNode(node));
                        }
                    }
                    catch (e_2_1) { e_2 = { error: e_2_1 }; }
                    finally {
                        try {
                            if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
                        }
                        finally { if (e_2) throw e_2.error; }
                    }
                    _this.path.pop();
                    _this.setCurrentNode(undefined);
                    _this.setParentNode(undefined);
                }, '}\n');
                break;
            }
            case constants_1.Syntax.ContainerGroup: {
                append(this.getGenerator('containerGroup').generate(item));
                break;
            }
            default:
                throw new Error("Node of type ".concat(item.type, " was not processed in main CodeGenerator class"));
        }
        return valueOf();
    };
    CodeGenerator.prototype.setCurrentNode = function (node) {
        this.currentNode = node;
    };
    /**
     * Generates an unique identifier according to the current parameter, namespace and container name. It is
     * safe to use this function and it'll always return the same value as long as you keep inside container's context.
     *
     * As you already know it's safety, you should always use `variant` to specify some specific information for that
     * variable whenever you'll be dealing with a different value and don't need that specific value to be overwritten
     * or to be conflicted with other variable identifiers
     *
     * TODO: check if worth the heavy work that needs to be executed so we can maintain the uniqueness of the returned
     * value. Other options are: use an incremental id and the difference will not be much as long as we keep the
     * same order of variables
     *
     * @param paramName current param name
     * @param variant optional parameter used to not generated the same hash, it should be used whenever the variable
     * is not carrying the parameter value
     */
    CodeGenerator.prototype.getVariableName = function (paramName, variant, includeComment) {
        var e_3, _a;
        if (includeComment === void 0) { includeComment = true; }
        var variableHash = '';
        var currentNode = this.currentNode;
        if (currentNode && currentNode.type === constants_1.Syntax.ContainerDeclaration) {
            var paramNode = void 0;
            try {
                for (var _b = (0, tslib_1.__values)(currentNode.body), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var node = _c.value;
                    if (node.type === constants_1.Syntax.ContainerParam && node.name === paramName) {
                        paramNode = node;
                        break;
                    }
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
                }
                finally { if (e_3) throw e_3.error; }
            }
            if (paramNode) {
                variableHash = [
                    "".concat(this.path.concat([paramName]).join('/'), ":"),
                    "".concat(this.translateParamType(paramNode.paramType)),
                    "(variant = ".concat(variant, ")")
                ].join(' ');
            }
            else {
                // console.warn(
                //     'getVariableName() was called to create a variable that does not belong to a param. ' +
                //     'Falling back on the random variable generation algorithm'
                // );
            }
        }
        if (!variableHash) {
            // console.warn('Generating random variable since it was called outside of container context');
            variableHash = (0, crypto_1.randomBytes)(4).toString('hex');
        }
        // console.log('Generating crc hash from "%s"', variableHash);
        var comment = '';
        if (includeComment) {
            comment = " /* ".concat(paramName, " */");
        }
        return "v_".concat(cyclic_rc_1["default"].crc_32(variableHash)).concat(comment);
    };
    CodeGenerator.prototype.getContainer = function (name) {
        var _this = this;
        var original = this.containers.find(function (container) { return container.name === _this.path.concat([name]).join('.'); });
        if (!original) {
            throw new Error("No original container generated for ".concat(this.path.concat([name]).join('.')));
        }
        return original;
    };
    CodeGenerator.prototype.getClassNameFromList = function (node) {
        if (node.type !== constants_1.Syntax.ContainerDeclaration && node.type !== constants_1.Syntax.ContainerGroup) {
            throw new Error('What else could supply a param type if not a container declaration or container group?');
        }
        if (node.type === constants_1.Syntax.ContainerGroup) {
            return "T".concat(this.getInterfaceName(node.name));
        }
        return this.getInterfaceName(node.name);
    };
    CodeGenerator.prototype.getNodeListFromParamExpression = function (node) {
        switch (node.type) {
            case constants_1.Syntax.Identifier: {
                for (var i = 0; i < this.path.length + 1; i++) {
                    var namespace = this.path.slice(0, this.path.length - i);
                    var results = this.resolveTypeInBody(node.value, this.nodes, namespace);
                    if (results.length === 0) {
                        continue;
                    }
                    return results;
                }
                return [];
            }
            case constants_1.Syntax.MemberExpression: {
                var name = node.right;
                var namespace = this.getNamespaceFromMemberExpression(node);
                var results = this.resolveTypeInBody(name, this.nodes, namespace.slice(0, namespace.length - 1));
                return results;
            }
        }
        return [];
    };
    CodeGenerator.prototype.getNamespaceFromMemberExpression = function (node, namespace) {
        if (namespace === void 0) { namespace = new Array(); }
        switch (node.type) {
            case constants_1.Syntax.MemberExpression:
                namespace = this.getNamespaceFromMemberExpression(node.left, namespace);
                return (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(namespace), false), [node.right], false);
            case constants_1.Syntax.Identifier:
                return (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(namespace), false), [node.value], false);
        }
        return namespace;
    };
    /**
     * Translate param type to a valid TypeScript type
     * @param node Node to be translated from
     */
    CodeGenerator.prototype.translateParamType = function (node) {
        switch (node.type) {
            case constants_1.Syntax.Identifier: {
                if (node.value === constants_1.Generics.String) {
                    return 'string';
                }
                else if (node.value === constants_1.Generics.Float || node.value === constants_1.Generics.Double) {
                    return 'number';
                }
                else if (node.value === constants_1.Generics.UInt32 || node.value === constants_1.Generics.Int32) {
                    return 'number';
                }
                else if (node.value === constants_1.Generics.UInt8 || node.value === constants_1.Generics.Int8) {
                    return 'number';
                }
                else if (node.value === constants_1.Generics.UInt16 || node.value === constants_1.Generics.Int16) {
                    return 'number';
                }
                else if (node.value === constants_1.Generics.UInt64 || node.value === constants_1.Generics.Int64) {
                    this.moduleResolver.requireImport('long', 'Long');
                    return 'Long';
                }
                else if (node.value === constants_1.Generics.Boolean) {
                    return 'boolean';
                }
                var results = this.getNodeListFromParamExpression(node);
                if (results.length === 0) {
                    throw new Error("No container name or type found for ".concat(node.value));
                }
                return this.getClassNameFromList(results[0]);
            }
            case constants_1.Syntax.MemberExpression: {
                var namespace = this.getNamespaceFromMemberExpression(node);
                var results = this.getNodeListFromParamExpression(node);
                if (results.length === 0) {
                    throw new Error("No container name or type found for ".concat(node.right, " (namespace = ").concat(namespace.join('.'), ")"));
                }
                for (var i = 0; i < namespace.length; i++) {
                    if (namespace[i] === this.path[i]) {
                        namespace.splice(i, 1);
                    }
                    else {
                        namespace.unshift.apply(namespace, (0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(this.prefixPackage), false));
                        break;
                    }
                }
                return namespace.slice(0, namespace.length - 1).concat([
                    this.getClassNameFromList(results[0])
                ]).join('.');
            }
            case constants_1.Syntax.Template:
                if (node.name === 'Vector') {
                    return "Array<".concat(this.translateParamType(node.arguments[0]), ">");
                }
                else if (node.name === 'TypedArray') {
                    var arrayOf = node.arguments[0];
                    if (arrayOf.type !== constants_1.Syntax.Identifier) {
                        throw new Error('TypedArray can contain only one argument and it must be an identifier');
                    }
                    if (arrayOf.value !== 'uint8') {
                        throw new Error('Current implementation only supports TypedArray<uint8>');
                    }
                    return 'Buffer';
                }
                else if (node.name === 'Optional') {
                    return "".concat(this.translateParamType(node.arguments[0]), " | undefined");
                }
                else if (node.name === 'StrictSize') {
                    return this.translateParamType(node.arguments[0]);
                }
                else if (node.name === 'Map') {
                    var key = this.translateParamType(node.arguments[0]);
                    var value = this.translateParamType(node.arguments[1]);
                    return "Map<".concat(key, ", ").concat(value, ">");
                }
                throw new Error("Unsupported template: ".concat(node.name));
        }
        throw new Error("Invalid param type: ".concat(node.type));
    };
    /**
     * Resolve a type inside a context
     * @param name Container name/container type to look for
     * @param body Body to look for
     * @param namespace Namespace to start from
     */
    CodeGenerator.prototype.resolveTypeInBody = function (name, body, namespace) {
        var e_4, _a, e_5, _b;
        var results = new Array();
        try {
            for (var body_1 = (0, tslib_1.__values)(body), body_1_1 = body_1.next(); !body_1_1.done; body_1_1 = body_1.next()) {
                var node = body_1_1.value;
                /**
                 * Since the `name` can't point to a namespace
                 * we have to ignore if this is not a container name or container group
                 */
                if (namespace.length === 0) {
                    if (node.type !== constants_1.Syntax.ContainerGroup && node.type !== constants_1.Syntax.ContainerDeclaration) {
                        continue;
                    }
                    if (node.type === constants_1.Syntax.ContainerGroup) {
                        if (node.name === name) {
                            results.push(node);
                            continue;
                        }
                        try {
                            for (var _c = (e_5 = void 0, (0, tslib_1.__values)(node.body)), _d = _c.next(); !_d.done; _d = _c.next()) {
                                var item = _d.value;
                                if (item.type === constants_1.Syntax.ContainerDeclaration && item.name === name) {
                                    results.push(item);
                                    break;
                                }
                            }
                        }
                        catch (e_5_1) { e_5 = { error: e_5_1 }; }
                        finally {
                            try {
                                if (_d && !_d.done && (_b = _c["return"])) _b.call(_c);
                            }
                            finally { if (e_5) throw e_5.error; }
                        }
                    }
                    else if (name === node.name) {
                        results.push(node);
                    }
                    continue;
                }
                else if (node.type === constants_1.Syntax.Namespace) {
                    if (node.name === namespace[0]) {
                        results.push.apply(results, (0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(this.resolveTypeInBody(name, node.body, namespace.slice(1))), false));
                        continue;
                    }
                }
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (body_1_1 && !body_1_1.done && (_a = body_1["return"])) _a.call(body_1);
            }
            finally { if (e_4) throw e_4.error; }
        }
        return results;
    };
    CodeGenerator.prototype.getCurrentPath = function () {
        return (0, tslib_1.__spreadArray)([], (0, tslib_1.__read)(this.path), false);
    };
    /**
     * Get package name according to current path
     */
    CodeGenerator.prototype.getPackageName = function () {
        return this.prefixPackage.concat(this.path).join('.');
    };
    CodeGenerator.prototype.getInterfaceName = function (name) {
        return name.split('').reduce(function (text, value, index) { return index === 0 ? value.toUpperCase() : text + value; }, '');
    };
    CodeGenerator.prototype.generate = function () {
        var e_6, _a;
        var _b = new code_stream_1["default"](this), write = _b.write, append = _b.append, prepend = _b.prepend, valueOf = _b.valueOf;
        append(this.getGenerator('utilClass').generate());
        append(this.getGenerator('dataContainer').generate());
        try {
            for (var _c = (0, tslib_1.__values)(this.nodes), _d = _c.next(); !_d.done; _d = _c.next()) {
                var node = _d.value;
                append(this.processNode(node));
            }
        }
        catch (e_6_1) { e_6 = { error: e_6_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c["return"])) _a.call(_c);
            }
            finally { if (e_6) throw e_6.error; }
        }
        prepend(this.getGenerator('importsDeclaration').generate());
        prepend('/* tslint:disable */\n');
        write('/* tslint:enable */\n');
        return valueOf();
    };
    CodeGenerator.prototype.getDefaultGenerators = function () {
        var generators = new Map();
        generators.set('containerGroup', new container_group_generator_1["default"](this))
            .set('containerDeclaration', new container_declaration_generator_1["default"](this))
            .set('utilClass', new util_class_code_generator_1["default"](this))
            .set('importsDeclaration', new imports_declaration_code_generator_1["default"](this))
            .set('dataContainer', new data_container_code_generator_1["default"](this));
        // Template generators
        generators
            .set('templateGenerator::Vector', new vector_code_generator_1["default"](this))
            .set('templateGenerator::Map', new map_code_generator_1["default"](this))
            .set('templateGenerator::TypedArray', new typed_array_code_generator_1["default"](this))
            .set('templateGenerator::Optional', new optional_code_generator_1["default"](this))
            .set('templateGenerator::StrictSize', new strict_size_code_generator_1["default"](this));
        return generators;
    };
    return CodeGenerator;
}());
exports["default"] = CodeGenerator;
