"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
/**
 * Processes decorators that are set up through
 * comments
 */
var CommentDecoratorProcessor = /** @class */ (function () {
    function CommentDecoratorProcessor(comments) {
        this.comments = comments;
        /**
         * Allowed characters in decorator or
         * decorator argument
         */
        this.allowedCharacters = /[a-zA-Z0-9\_\.]/;
        this.offset = 0;
    }
    CommentDecoratorProcessor.prototype.find = function () {
        var e_1, _a;
        var decorators = new Array();
        var _b = this, comments = _b.comments, allowedCharacters = _b.allowedCharacters;
        try {
            for (var comments_1 = (0, tslib_1.__values)(comments), comments_1_1 = comments_1.next(); !comments_1_1.done; comments_1_1 = comments_1.next()) {
                var comment = comments_1_1.value;
                while (this.offset < comment.length) {
                    this.offset = comment.indexOf('@', this.offset);
                    if (this.offset === -1) {
                        break;
                    }
                    ++this.offset;
                    var decorator = '';
                    var i = void 0;
                    for (i = this.offset; i < comment.length; i++) {
                        if (!allowedCharacters.test(comment[i])) {
                            decorator = comment.substring(this.offset, i);
                            break;
                        }
                    }
                    this.offset = i;
                    var args = new Array();
                    for (i = this.offset; i < comment.length; i++) {
                        var endOfString = i === (comment.length - 1);
                        if (!allowedCharacters.test(comment[i]) || endOfString) {
                            if (endOfString) {
                                ++i;
                            }
                            var arg = comment.substring(this.offset, i);
                            if (arg.trim() !== '') {
                                args.push(arg.trim());
                            }
                            this.offset = i;
                        }
                        var ch = comment.charCodeAt(i);
                        if (this.chIsEndOfLine(ch)) {
                            i++;
                            break;
                        }
                    }
                    decorators.push({
                        name: decorator,
                        args: args
                    });
                    this.offset = i;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (comments_1_1 && !comments_1_1.done && (_a = comments_1["return"])) _a.call(comments_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return decorators;
    };
    CommentDecoratorProcessor.prototype.chIsEndOfLine = function (ch) {
        if (ch === 10) { // \n Line break
            return true;
        }
        return false;
    };
    return CommentDecoratorProcessor;
}());
exports["default"] = CommentDecoratorProcessor;
