"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var constants_1 = require("../ast-parser/constants");
var code_generator_child_1 = (0, tslib_1.__importDefault)(require("./code-generator-child"));
var generic_code_generator_1 = (0, tslib_1.__importDefault)(require("./generic-code-generator"));
var code_stream_1 = (0, tslib_1.__importDefault)(require("./code-stream"));
/**
 * Responsible for generating container declaration code
 */
var ContainerDeclarationGenerator = /** @class */ (function (_super) {
    (0, tslib_1.__extends)(ContainerDeclarationGenerator, _super);
    function ContainerDeclarationGenerator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ContainerDeclarationGenerator.prototype.generate = function (item) {
        var e_1, _a;
        var _this = this;
        this.setCurrentNode(item);
        var lastParam;
        try {
            for (var _b = (0, tslib_1.__values)(item.body), _c = _b.next(); !_c.done; _c = _b.next()) {
                var node = _c.value;
                if (node.type === constants_1.Syntax.ContainerParam) {
                    lastParam = node;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
            }
            finally { if (e_1) throw e_1.error; }
        }
        var container = this.getContainer(item.name);
        var parentNode = this.getParentNode();
        var _d = this.cs, write = _d.write, append = _d.append, valueOf = _d.valueOf;
        if (!parentNode || parentNode.type !== constants_1.Syntax.ContainerGroup) {
            throw new Error('Parent node of a container declaration must always be a container group');
        }
        this.createContainerParamsInterface(item);
        this.writeContainerClassComments(item);
        var interfaceName = this.getInterfaceName(item.name);
        var parentInterfaceName = this.getInterfaceName(parentNode.name);
        write("export class ".concat(interfaceName, " extends T").concat(parentInterfaceName, " implements I").concat(interfaceName, "Params {\n"), function () {
            append(_this.createContainerInterfaceParams(item, function (c) {
                return "public readonly ".concat(c);
            }));
            write('constructor(');
            if (lastParam) {
                append("params: I".concat(interfaceName, "Params"));
            }
            append(')\n');
            write('{\n', function () {
                var e_2, _a;
                write("super(".concat(container.id, ", \"").concat(container.name, "\");\n"));
                try {
                    for (var _b = (0, tslib_1.__values)(container.params), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var param = _c.value;
                        write("this.".concat(param.name, " = params.").concat(param.name, ";\n"));
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            }, '}\n');
            _this.getModuleResolver().requireImport('message-ff', {
                args: ['Serializer', 'Deserializer']
            });
            /* container body */
            write("public static decode(deserializer: Deserializer, ignoreHeader = false): ".concat(interfaceName, " {\n"), function () { return append(_this.createDecodingStatements(item)); }, '}\n');
            write('public encode(serializer: Serializer, ignoreHeader = false): void {\n', function () { return append(_this.createEncodingStatements(item)); }, '}\n');
            write("public copy(".concat(lastParam ? "params: Partial<I".concat(interfaceName, "Params>") : '', "): ").concat(interfaceName, " {\n"), function () { return append(_this.createCopyStatements(item)); }, '}\n');
        }, '}\n');
        this.setCurrentNode(undefined);
        return valueOf();
    };
    ContainerDeclarationGenerator.prototype.attachComments = function (comments) {
        var e_3, _a, e_4, _b;
        var _c = new code_stream_1["default"](this), write = _c.write, valueOf = _c.valueOf;
        if (comments.length === 0)
            return '';
        try {
            for (var comments_1 = (0, tslib_1.__values)(comments), comments_1_1 = comments_1.next(); !comments_1_1.done; comments_1_1 = comments_1.next()) {
                var comment = comments_1_1.value;
                switch (comment.type) {
                    case constants_1.Syntax.SingleLineComment:
                        write("// ".concat(comment.value, "\n"));
                        break;
                    case constants_1.Syntax.MultiLineComment: {
                        write("/**\n");
                        var lines = comment.value.split('\n');
                        try {
                            for (var lines_1 = (e_4 = void 0, (0, tslib_1.__values)(lines)), lines_1_1 = lines_1.next(); !lines_1_1.done; lines_1_1 = lines_1.next()) {
                                var line = lines_1_1.value;
                                if (!line.trim()) {
                                    continue;
                                }
                                write(" ".concat(line.replace(/^(\ )+/, ''), "\n"));
                            }
                        }
                        catch (e_4_1) { e_4 = { error: e_4_1 }; }
                        finally {
                            try {
                                if (lines_1_1 && !lines_1_1.done && (_b = lines_1["return"])) _b.call(lines_1);
                            }
                            finally { if (e_4) throw e_4.error; }
                        }
                        write(' */\n');
                    }
                }
            }
        }
        catch (e_3_1) { e_3 = { error: e_3_1 }; }
        finally {
            try {
                if (comments_1_1 && !comments_1_1.done && (_a = comments_1["return"])) _a.call(comments_1);
            }
            finally { if (e_3) throw e_3.error; }
        }
        return valueOf();
    };
    ContainerDeclarationGenerator.prototype.createCopyStatements = function (item) {
        var e_5, _a, e_6, _b;
        var _this = this;
        var _c = new code_stream_1["default"](this), write = _c.write, valueOf = _c.valueOf;
        var lastParam;
        try {
            for (var _d = (0, tslib_1.__values)(item.body), _e = _d.next(); !_e.done; _e = _d.next()) {
                var node = _e.value;
                if (node.type !== constants_1.Syntax.ContainerParam) {
                    continue;
                }
                lastParam = node;
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_a = _d["return"])) _a.call(_d);
            }
            finally { if (e_5) throw e_5.error; }
        }
        if (lastParam) {
            write('let changed = false;\n');
            try {
                for (var _f = (0, tslib_1.__values)(item.body), _g = _f.next(); !_g.done; _g = _f.next()) {
                    var node = _g.value;
                    if (node.type !== constants_1.Syntax.ContainerParam) {
                        continue;
                    }
                    write("if(!changed && this.".concat(node.name, " !== params.").concat(node.name, ") changed = true;\n"));
                }
            }
            catch (e_6_1) { e_6 = { error: e_6_1 }; }
            finally {
                try {
                    if (_g && !_g.done && (_b = _f["return"])) _b.call(_f);
                }
                finally { if (e_6) throw e_6.error; }
            }
            write('if(changed) {\n', function () {
                write("return new ".concat(_this.getInterfaceName(item.name), "({\n"), function () {
                    write("...this,\n");
                    write("...params\n");
                }, "});\n");
            }, '}\n');
        }
        write('return this;\n');
        return valueOf();
    };
    ContainerDeclarationGenerator.prototype.createContainerParamsInterface = function (item) {
        var _this = this;
        var _a = this.cs, write = _a.write, append = _a.append;
        var interfaceName = "I".concat(this.getInterfaceName(item.name), "Params");
        write("interface ".concat(interfaceName, " {\n"), function () {
            append(_this.createContainerInterfaceParams(item));
        }, '}\n');
    };
    ContainerDeclarationGenerator.prototype.createContainerInterfaceParams = function (item, transform) {
        var e_7, _a;
        if (transform === void 0) { transform = function (v) { return v; }; }
        var optional = '';
        var _b = new code_stream_1["default"](this), write = _b.write, append = _b.append, valueOf = _b.valueOf;
        try {
            for (var _c = (0, tslib_1.__values)(item.body), _d = _c.next(); !_d.done; _d = _c.next()) {
                var node = _d.value;
                if (node.type !== constants_1.Syntax.ContainerParam) {
                    continue;
                }
                if (node.leadingComments.length > 0) {
                    append(this.attachComments(node.leadingComments));
                }
                var paramType = void 0;
                if (node.paramType.type === constants_1.Syntax.Template && node.paramType.name === 'Optional') {
                    optional = '?';
                    paramType = this.translateParamType(node.paramType.arguments[0]);
                }
                else {
                    optional = '';
                    paramType = this.translateParamType(node.paramType);
                }
                write(transform("".concat(node.name).concat(optional, ": ").concat(paramType, ";\n")));
            }
        }
        catch (e_7_1) { e_7 = { error: e_7_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c["return"])) _a.call(_c);
            }
            finally { if (e_7) throw e_7.error; }
        }
        return valueOf();
    };
    ContainerDeclarationGenerator.prototype.getDecoratorFromParam = function (node) {
        if (node.type !== constants_1.Syntax.ContainerParam) {
            return '';
        }
        return '';
    };
    ContainerDeclarationGenerator.prototype.createDecodingStatements = function (node) {
        var e_8, _a, e_9, _b;
        if (node.type !== constants_1.Syntax.ContainerDeclaration) {
            throw new TypeError('Node must be a container declaration');
        }
        var _c = new code_stream_1["default"](this), write = _c.write, valueOf = _c.valueOf, append = _c.append;
        var container = this.getContainer(node.name);
        write('if(!ignoreHeader) {\n', function () {
            write('const id = deserializer.readUInt32();\n');
            write("if(".concat(container.id, " !== id) {\n"), function () {
                write("throw new Error(\n", function () {
                    write("`Invalid container id: Expected ".concat(container.id, " but received ${id} instead.`\n"));
                }, ');\n');
            }, '}\n');
        }, '}\n');
        var paramsValues = new Map();
        try {
            for (var _d = (0, tslib_1.__values)(node.body), _e = _d.next(); !_e.done; _e = _d.next()) {
                var item = _e.value;
                if (item.type !== constants_1.Syntax.ContainerParam) {
                    continue;
                }
                paramsValues.set(item.name, this.getVariableName(item.name));
                write("let ".concat(paramsValues.get(item.name), ": ").concat(this.translateParamType(item.paramType)));
                var defaultValue = this.getParamDefaultFromType(item.paramType);
                if (defaultValue) {
                    append(" = ".concat(defaultValue, ";\n"));
                }
                else {
                    append(';\n');
                }
            }
        }
        catch (e_8_1) { e_8 = { error: e_8_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_a = _d["return"])) _a.call(_d);
            }
            finally { if (e_8) throw e_8.error; }
        }
        var lastParam;
        try {
            for (var _f = (0, tslib_1.__values)(node.body), _g = _f.next(); !_g.done; _g = _f.next()) {
                var item = _g.value;
                if (item.type !== constants_1.Syntax.ContainerParam) {
                    continue;
                }
                var assignmentVariable = paramsValues.get(item.name);
                if (!assignmentVariable) {
                    throw new Error("Could not find variable name for variable ".concat(item.name));
                }
                append(this.createParamDecodingStatement(item, {
                    assignmentVariable: assignmentVariable
                }));
                lastParam = item;
            }
        }
        catch (e_9_1) { e_9 = { error: e_9_1 }; }
        finally {
            try {
                if (_g && !_g.done && (_b = _f["return"])) _b.call(_f);
            }
            finally { if (e_9) throw e_9.error; }
        }
        if (lastParam) {
            write("return new ".concat(this.getClassNameFromList(node), "({\n"), function () {
                var e_10, _a;
                try {
                    for (var _b = (0, tslib_1.__values)(node.body), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var item = _c.value;
                        if (item.type !== constants_1.Syntax.ContainerParam) {
                            continue;
                        }
                        write("\"".concat(item.name, "\": ").concat(paramsValues.get(item.name)));
                        if (lastParam !== item) {
                            append(',\n');
                        }
                        else {
                            append('\n');
                        }
                    }
                }
                catch (e_10_1) { e_10 = { error: e_10_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
                    }
                    finally { if (e_10) throw e_10.error; }
                }
            }, '});\n');
        }
        else {
            write("return new ".concat(this.getClassNameFromList(node), "();\n"));
        }
        return valueOf();
    };
    ContainerDeclarationGenerator.prototype.createEncodingStatements = function (item) {
        var e_11, _a;
        var original = this.getContainer(item.name);
        var _b = new code_stream_1["default"](this), append = _b.append, write = _b.write, valueOf = _b.valueOf;
        write("if(ignoreHeader != true) serializer.writeUInt32(".concat(original.id, ");\n"));
        try {
            for (var _c = (0, tslib_1.__values)(item.body), _d = _c.next(); !_d.done; _d = _c.next()) {
                var node = _d.value;
                if (node.type !== constants_1.Syntax.ContainerParam) {
                    continue;
                }
                append(this.createParamEncodingStatement(node, {
                    assignmentVariable: "this.".concat(node.name)
                }));
            }
        }
        catch (e_11_1) { e_11 = { error: e_11_1 }; }
        finally {
            try {
                if (_d && !_d.done && (_a = _c["return"])) _a.call(_c);
            }
            finally { if (e_11) throw e_11.error; }
        }
        return valueOf();
    };
    ContainerDeclarationGenerator.prototype.createParamDecodingStatement = function (node, options) {
        if (node.type !== constants_1.Syntax.ContainerParam) {
            throw new TypeError('Node must be container param type');
        }
        var paramType = node.paramType;
        if (paramType.type === constants_1.Syntax.Identifier) {
            var contents = new generic_code_generator_1["default"](this).getDecodingCode(paramType, options);
            if (contents) {
                return contents;
            }
        }
        else if (paramType.type === constants_1.Syntax.Template) {
            var typeGenerator = this.getGenerator("templateGenerator::".concat(paramType.name));
            return typeGenerator.getDecodingCode(node, options);
        }
        var assignmentVariable = options.assignmentVariable;
        return this.indentCode("".concat(assignmentVariable, " = ").concat(this.translateParamType(node.paramType), ".decode(deserializer);\n"));
    };
    ContainerDeclarationGenerator.prototype.createToplevelDecodeMethod = function (node) {
        var _this = this;
        if (node.type !== constants_1.Syntax.ContainerGroup) {
            throw new Error('Node must be container group type');
        }
        var _a = new code_stream_1["default"](this), write = _a.write, valueOf = _a.valueOf;
        write("public static decode(deserializer: Deserializer): ".concat(this.getClassNameFromList(node), " {\n"), function () {
            var e_12, _a;
            write('const id = deserializer.readUInt32();\n');
            var ids = new Array();
            var _loop_1 = function (item) {
                if (item.type !== constants_1.Syntax.ContainerDeclaration) {
                    return "continue";
                }
                var container = _this.getContainer(item.name);
                ids.push(container.id);
                write("if(id === 0x".concat(container.id.toString(16), ") {\n"), function () {
                    write("return ".concat(_this.getClassNameFromList(item), ".decode(deserializer, true);\n"));
                }, '}\n');
            };
            try {
                for (var _b = (0, tslib_1.__values)(node.body), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var item = _c.value;
                    _loop_1(item);
                }
            }
            catch (e_12_1) { e_12 = { error: e_12_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b["return"])) _a.call(_b);
                }
                finally { if (e_12) throw e_12.error; }
            }
            write("throw new Error(`Expected one of ".concat(ids.join(' / '), " ids but got ${id} instead`);\n"));
        }, '}\n');
        return valueOf();
    };
    ContainerDeclarationGenerator.prototype.createParamEncodingStatement = function (node, options) {
        if (node.type !== constants_1.Syntax.ContainerParam) {
            throw new TypeError('Node must be container param type');
        }
        var assignmentVariable = options.assignmentVariable;
        var paramType = node.paramType;
        var _a = new code_stream_1["default"](this), write = _a.write, valueOf = _a.valueOf;
        if (paramType.type === constants_1.Syntax.Identifier) {
            var contents = new generic_code_generator_1["default"](this).getEncodingCode(paramType, options);
            if (contents) {
                return contents;
            }
        }
        else if (paramType.type === constants_1.Syntax.Template) {
            var typeGenerator = this.getGenerator("templateGenerator::".concat(paramType.name));
            return typeGenerator.getEncodingCode(node, options);
        }
        write("".concat(assignmentVariable, ".encode(serializer, false);\n"));
        return valueOf();
    };
    ContainerDeclarationGenerator.prototype.writeContainerClassComments = function (item) {
        var append = this.cs.append;
        if (!item.leadingComments.length) {
            return;
        }
        append(this.attachComments(item.leadingComments));
    };
    return ContainerDeclarationGenerator;
}(code_generator_child_1["default"]));
exports["default"] = ContainerDeclarationGenerator;
