"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var constants_1 = require("../ast-parser/constants");
var type_code_generator_1 = (0, tslib_1.__importDefault)(require("./type-code-generator"));
/**
 * Class responsible for generating code related
 * to generic types
 */
var GenericCodeGenerator = /** @class */ (function (_super) {
    (0, tslib_1.__extends)(GenericCodeGenerator, _super);
    function GenericCodeGenerator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    GenericCodeGenerator.prototype.getEncodingCode = function (paramType, _a) {
        var assignmentVariable = _a.assignmentVariable;
        var _b = this.cs, write = _b.write, valueOf = _b.valueOf;
        if (paramType.value === constants_1.Generics.String) {
            write("serializer.writeString(".concat(assignmentVariable, ");\n"));
        }
        else if (paramType.value === constants_1.Generics.UInt32) {
            write("serializer.writeUInt32(".concat(assignmentVariable, ");\n"));
        }
        else if (paramType.value === constants_1.Generics.Double) {
            write("serializer.writeDouble(".concat(assignmentVariable, ");\n"));
        }
        else if (paramType.value === constants_1.Generics.Int32) {
            write("serializer.writeInt32(".concat(assignmentVariable, ");\n"));
        }
        else if (paramType.value === constants_1.Generics.Boolean) {
            write("serializer.writeUInt8(".concat(assignmentVariable, " === true ? 1 : 0);\n"));
        }
        else if (paramType.value === constants_1.Generics.UInt16) {
            write("serializer.writeUInt16(".concat(assignmentVariable, ");\n"));
        }
        else if (paramType.value === constants_1.Generics.UInt64) {
            write("serializer.writeUInt64(".concat(assignmentVariable, ");\n"));
        }
        else if (paramType.value === constants_1.Generics.Int64) {
            write("serializer.writeInt64(".concat(assignmentVariable, ");\n"));
        }
        return valueOf();
    };
    GenericCodeGenerator.prototype.getDecodingCode = function (paramType, _a) {
        var assignmentVariable = _a.assignmentVariable;
        var _b = this.cs, write = _b.write, valueOf = _b.valueOf;
        if (paramType.value === constants_1.Generics.String) {
            write("".concat(assignmentVariable, " = deserializer.readBuffer(deserializer.readUInt32()).toString('utf8');\n"));
        }
        else if (paramType.value === constants_1.Generics.UInt32) {
            write("".concat(assignmentVariable, " = deserializer.readUInt32();\n"));
        }
        else if (paramType.value === constants_1.Generics.Double) {
            write("".concat(assignmentVariable, " = deserializer.readDouble();\n"));
        }
        else if (paramType.value === constants_1.Generics.Int32) {
            write("".concat(assignmentVariable, " = deserializer.readInt32();\n"));
        }
        else if (paramType.value === constants_1.Generics.Boolean) {
            write("".concat(assignmentVariable, " = deserializer.readUInt8() ? true : false;\n"));
        }
        else if (paramType.value === constants_1.Generics.UInt16) {
            write("".concat(assignmentVariable, " = deserializer.readUInt16();\n"));
        }
        else if (paramType.value === constants_1.Generics.UInt64) {
            write("".concat(assignmentVariable, " = deserializer.readUInt64();\n"));
        }
        else if (paramType.value === constants_1.Generics.Int64) {
            write("".concat(assignmentVariable, " = deserializer.readInt64();\n"));
        }
        return valueOf();
    };
    return GenericCodeGenerator;
}(type_code_generator_1["default"]));
exports["default"] = GenericCodeGenerator;
