"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var template_code_generator_1 = (0, tslib_1.__importDefault)(require("./template-code-generator"));
var constants_1 = require("../../ast-parser/constants");
var MapCodeGenerator = /** @class */ (function (_super) {
    (0, tslib_1.__extends)(MapCodeGenerator, _super);
    function MapCodeGenerator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MapCodeGenerator.prototype.getDefaultValueExpression = function () {
        return 'new Map()';
    };
    MapCodeGenerator.prototype.getEncodingCode = function (node, options) {
        var _this = this;
        var _a = this.cs, valueOf = _a.valueOf, append = _a.append, write = _a.write;
        var paramType = node.paramType;
        if (paramType.type !== constants_1.Syntax.Template) {
            throw new Error('Invalid param type for Map code generator');
        }
        write("serializer.writeUInt32(".concat(options.assignmentVariable, ".size);\n"));
        write("for(const [key, value] of ".concat(options.assignmentVariable, ") {\n"), function () {
            append(_this.getContainerDeclarationGenerator().createParamEncodingStatement((0, tslib_1.__assign)((0, tslib_1.__assign)({}, node), { paramType: paramType.arguments[0], type: constants_1.Syntax.ContainerParam }), {
                assignmentVariable: 'key'
            }));
            append(_this.getContainerDeclarationGenerator().createParamEncodingStatement((0, tslib_1.__assign)((0, tslib_1.__assign)({}, node), { paramType: paramType.arguments[1], type: constants_1.Syntax.ContainerParam }), {
                assignmentVariable: 'value'
            }));
        }, '}\n');
        return valueOf();
    };
    MapCodeGenerator.prototype.getDecodingCode = function (node, options) {
        var _this = this;
        var _a = this.cs, valueOf = _a.valueOf, write = _a.write, append = _a.append;
        var paramType = node.paramType;
        if (paramType.type !== constants_1.Syntax.Template) {
            throw new Error('Invalid param type for Map code generator');
        }
        var lengthVariable = this.getVariableName(node.name, 'map length');
        var keyVariable = this.getVariableName(node.name, 'map key');
        var valueVariable = this.getVariableName(node.name, 'map value');
        write("let ".concat(lengthVariable, " = deserializer.readUInt32();\n"));
        write("let ".concat(keyVariable, ": ").concat(this.translateParamType(paramType.arguments[0]), ";\n"));
        write("let ".concat(valueVariable, ": ").concat(this.translateParamType(paramType.arguments[1]), ";\n"));
        write("while(".concat(lengthVariable, " > 0) {\n"), function () {
            append(_this.getContainerDeclarationGenerator().createParamDecodingStatement((0, tslib_1.__assign)((0, tslib_1.__assign)({}, node), { paramType: paramType.arguments[0], type: constants_1.Syntax.ContainerParam }), {
                assignmentVariable: keyVariable
            }));
            append(_this.getContainerDeclarationGenerator().createParamDecodingStatement((0, tslib_1.__assign)((0, tslib_1.__assign)({}, node), { paramType: paramType.arguments[1], type: constants_1.Syntax.ContainerParam }), {
                assignmentVariable: valueVariable
            }));
            write("".concat(options.assignmentVariable, ".set(").concat(keyVariable, ", ").concat(valueVariable, ");\n"));
            write("--".concat(lengthVariable, ";\n"));
        }, '}\n');
        return valueOf();
    };
    return MapCodeGenerator;
}(template_code_generator_1["default"]));
exports["default"] = MapCodeGenerator;
