"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var constants_1 = require("../../ast-parser/constants");
var template_code_generator_1 = (0, tslib_1.__importDefault)(require("./template-code-generator"));
var StrictSizeCodeGenerator = /** @class */ (function (_super) {
    (0, tslib_1.__extends)(StrictSizeCodeGenerator, _super);
    function StrictSizeCodeGenerator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    StrictSizeCodeGenerator.prototype.getDefaultValueExpression = function (node) {
        return this.getParamDefaultFromType(node.arguments[0]);
    };
    StrictSizeCodeGenerator.prototype.getDecodingCode = function (node, options) {
        var _a = this.cs, valueOf = _a.valueOf, append = _a.append;
        var paramType = node.paramType;
        if (paramType.type !== constants_1.Syntax.Template) {
            throw new Error('Received invalid param type');
        }
        var _b = (0, tslib_1.__read)(paramType.arguments, 1), firstArgument = _b[0];
        append(this.getContainerDeclarationGenerator().createParamDecodingStatement((0, tslib_1.__assign)((0, tslib_1.__assign)({}, node), { paramType: firstArgument }), options));
        this.createValidationCode(node, paramType, options);
        return valueOf();
    };
    StrictSizeCodeGenerator.prototype.getEncodingCode = function (node, options) {
        var _a = this.cs, valueOf = _a.valueOf, append = _a.append;
        var paramType = node.paramType;
        if (paramType.type !== constants_1.Syntax.Template) {
            throw new Error('Invalid param type');
        }
        this.createValidationCode(node, paramType, options);
        append(this.getContainerDeclarationGenerator().createParamEncodingStatement((0, tslib_1.__assign)((0, tslib_1.__assign)({}, node), { paramType: paramType.arguments[0] }), options));
        return valueOf();
    };
    StrictSizeCodeGenerator.prototype.createValidationCode = function (node, paramType, options) {
        var write = this.cs.write;
        var _a = (0, tslib_1.__read)(paramType.arguments, 2), firstArgument = _a[0], secondArgument = _a[1];
        if (secondArgument.type !== constants_1.Syntax.LiteralNumber) {
            throw new Error('Second argument of StrictSize must be of type number');
        }
        var size = secondArgument.value;
        if (firstArgument.type !== constants_1.Syntax.Identifier) {
            throw new Error('First argument must be identifier');
        }
        var lengthVariable = this.getVariableName(node.name, 'length variable');
        write("let ".concat(lengthVariable, ": number;"));
        switch (firstArgument.value) {
            case constants_1.Generics.String:
                write("".concat(lengthVariable, " = ").concat(options.assignmentVariable, ".length;\n"));
                break;
            default:
                throw new Error('Invalid first argument type');
        }
        write("if(".concat(lengthVariable, " !== ").concat(size, ") {\n"), function () {
            write("throw new Error(`Expected ".concat(size, " but got ${valueVariable.length} instead`);\n"));
        }, '}\n');
    };
    return StrictSizeCodeGenerator;
}(template_code_generator_1["default"]));
exports["default"] = StrictSizeCodeGenerator;
