"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var constants_1 = require("../../ast-parser/constants");
var template_code_generator_1 = (0, tslib_1.__importDefault)(require("./template-code-generator"));
var VectorCodeGenerator = /** @class */ (function (_super) {
    (0, tslib_1.__extends)(VectorCodeGenerator, _super);
    function VectorCodeGenerator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    VectorCodeGenerator.prototype.getDecodingCode = function (node, options) {
        var _this = this;
        var _a = this.cs, write = _a.write, valueOf = _a.valueOf, append = _a.append;
        var paramType = node.paramType;
        if (paramType.type !== constants_1.Syntax.Template) {
            throw new Error('Param type must always be a template');
        }
        var lengthVariable = this.getVariableName(node.name, 'vector length');
        var outputVariable = this.getVariableName(node.name, 'vector item value');
        write("let ".concat(lengthVariable, " = deserializer.readUInt32();\n"));
        write("let ".concat(outputVariable, ": ").concat(this.translateParamType(paramType.arguments[0]), ";\n"));
        write("while(".concat(lengthVariable, " > 0) {\n"), function () {
            append(_this.getContainerDeclarationGenerator().createParamDecodingStatement((0, tslib_1.__assign)((0, tslib_1.__assign)({}, node), { type: constants_1.Syntax.ContainerParam, name: node.name, paramType: paramType.arguments[0] }), {
                assignmentVariable: outputVariable
            }));
            write("".concat(options.assignmentVariable, ".push(").concat(outputVariable, ");\n"));
            write("--".concat(lengthVariable, ";\n"));
        }, '}\n');
        return valueOf();
    };
    VectorCodeGenerator.prototype.getEncodingCode = function (node, options) {
        var _this = this;
        var _a = this.cs, write = _a.write, append = _a.append, valueOf = _a.valueOf;
        var assignmentVariable = options.assignmentVariable;
        var paramType = node.paramType;
        if (paramType.type !== constants_1.Syntax.Template) {
            throw new Error('Invalid param type');
        }
        write("serializer.writeUInt32(".concat(assignmentVariable, ".length);\n"));
        write("for(const item of ".concat(assignmentVariable, ") {\n"), function () {
            append(_this.getContainerDeclarationGenerator().createParamEncodingStatement((0, tslib_1.__assign)((0, tslib_1.__assign)({}, node), { type: constants_1.Syntax.ContainerParam, paramType: paramType.arguments[0] }), { assignmentVariable: 'item' }));
        }, '}\n');
        return valueOf();
    };
    VectorCodeGenerator.prototype.getDefaultValueExpression = function (node) {
        return "new Array<".concat(this.translateParamType(node.arguments[0]), ">()");
    };
    return VectorCodeGenerator;
}(template_code_generator_1["default"]));
exports["default"] = VectorCodeGenerator;
