"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var long_1 = (0, tslib_1.__importDefault)(require("long"));
var Deserializer = /** @class */ (function () {
    function Deserializer(buffer, offset) {
        if (offset === void 0) { offset = 0; }
        this.offset = 0;
        this.buffer = buffer;
        this.offset = offset;
    }
    Deserializer.prototype.readUInt8 = function () {
        var value = this.buffer.readUInt8(this.offset);
        this.offset += 1;
        return value;
    };
    Deserializer.prototype.readUInt64 = function () {
        var low = this.readUInt32();
        var high = this.readUInt32();
        return new long_1["default"](low, high, true);
    };
    Deserializer.prototype.readInt64 = function () {
        var low = this.readInt32();
        var high = this.readInt32();
        return new long_1["default"](low, high, false);
    };
    Deserializer.prototype.readInt8 = function () {
        var value = this.buffer.readInt8(this.offset);
        this.offset += 1;
        return value;
    };
    Deserializer.prototype.readFloat = function () {
        var value = this.buffer.readFloatLE(this.offset);
        this.offset += 4;
        return value;
    };
    Deserializer.prototype.readDouble = function () {
        var value = this.buffer.readDoubleLE(this.offset);
        this.offset += 8;
        return value;
    };
    Deserializer.prototype.readBuffer = function (length) {
        var result = this.buffer.slice(this.offset, this.offset + length);
        this.offset += length;
        return result;
    };
    Deserializer.prototype.readBoolean = function () {
        var value = this.readUInt8();
        if (value == 1)
            return true;
        else if (value == 0)
            return false;
        console.warn('WARNING: Got unexpected value for boolean. Expected 1 or 0 but got %d instead', value);
        return value ? true : false;
    };
    Deserializer.prototype.readUInt32 = function () {
        var result = this.buffer.readUInt32LE(this.offset);
        this.offset += 4;
        return result;
    };
    Deserializer.prototype.readInt32 = function () {
        var result = this.buffer.readInt32LE(this.offset);
        this.offset += 4;
        return result;
    };
    Deserializer.prototype.readInt16 = function () {
        var result = this.buffer.readInt16LE(this.offset);
        this.offset += 2;
        return result;
    };
    Deserializer.prototype.readUInt16 = function () {
        var result = this.buffer.readUInt16LE(this.offset);
        this.offset += 2;
        return result;
    };
    return Deserializer;
}());
exports["default"] = Deserializer;
