/// <reference types="node" />
import { Container, Generics } from '../ast-parser/constants';
import { ParamTemplate, Param } from '../ast-parser/param';
import Serializer from './serializer';
import Deserializer from './deserializer';
import { PropertyType } from './template-processor';
import TemplateProcessor from './template-processor';
import Long from 'long';
export interface SchemaOptions {
    templateProcessors?: Map<string, TemplateProcessor>;
}
declare class Schema {
    containers: Map<string, Container>;
    containersCRC: Map<number, Container>;
    templateProcessors: Map<string, TemplateProcessor>;
    constructor(containers: Container[], options?: SchemaOptions);
    decodeGeneric(des: Deserializer, type: Generics): string | number | boolean | Long;
    encodeGeneric(serializer: Serializer, type: string, value: any): void;
    /**
     * Get container name from object.
     * Generally defined as [$container_name, $params]
     */
    getContainerName(value: any): string;
    getContainerParams(value: any): any;
    createObject(container: Container, params: any): any;
    decodeTemplate(deserializer: Deserializer, template: ParamTemplate, result: any, prop: PropertyType): void;
    decode(deserializer: Buffer | Deserializer): any;
    decodeContainerParam(deserializer: Deserializer, param: Param, result: any, prop: PropertyType): any;
    encodeReference(serializer: Serializer, containers: string[], value: any): void;
    encodeTemplate(serializer: Serializer, param: ParamTemplate, value: any): void;
    /**
     * Return default value for generics
     */
    getGenericDefault(value: Generics): any;
    validateGeneric(type: Generics, value: any): void;
    encodeContainerParam(serializer: Serializer, param: Param, value: any): void;
    encode(name: any, props?: undefined | Serializer): Buffer;
    encode(name: string, props: any, serializer?: Serializer): Buffer;
}
export default Schema;
