"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var template_processor_1 = (0, tslib_1.__importDefault)(require("./template-processor"));
var constants_1 = require("../ast-parser/constants");
var utilities_1 = require("./utilities");
var StrictSizeProcessor = /** @class */ (function (_super) {
    (0, tslib_1.__extends)(StrictSizeProcessor, _super);
    function StrictSizeProcessor() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    StrictSizeProcessor.prototype.validate = function (result, length) {
        var valid = false;
        if (Array.isArray(result) || typeof result === 'string') {
            valid = result.length === length;
        }
        else if (typeof result === 'number') {
            valid = result === length;
        }
        else if ((0, utilities_1.isTypedArray)(result) || Buffer.isBuffer(result)) {
            valid = result.byteLength === length;
        }
        return valid;
    };
    StrictSizeProcessor.prototype.encode = function (serializer, args, input) {
        var firstArgument = args[0];
        var secondArgument = args[1];
        var length = 0;
        if (firstArgument.type === constants_1.Syntax.LiteralNumber) {
            throw new Error("Invalid value for first argument");
        }
        else if (secondArgument.type === constants_1.Syntax.LiteralNumber) {
            length = secondArgument.value;
        }
        else {
            throw new Error("Invalid value for second argument, must be a literal number");
        }
        if (!this.validate(input, length)) {
            throw new Error("Received invalid size for strict-sized param");
        }
        this.schema.encodeContainerParam(serializer, firstArgument, input);
    };
    StrictSizeProcessor.prototype.decode = function (deserializer, args, result, prop) {
        var firstArgument = args[0];
        var secondArgument = args[1];
        var length = 0;
        if (firstArgument.type === constants_1.Syntax.LiteralNumber) {
            throw new Error("Invalid first argument for strict size template");
        }
        else if (secondArgument.type === constants_1.Syntax.LiteralNumber) {
            length = secondArgument.value;
        }
        else {
            throw new Error("Invalid value for second argument, must be a literal number");
        }
        this.schema.decodeContainerParam(deserializer, firstArgument, result, prop);
        if (!this.validate(result[prop], length)) {
            throw new Error("Received invalid size for strict-sized param");
        }
    };
    return StrictSizeProcessor;
}(template_processor_1["default"]));
exports["default"] = StrictSizeProcessor;
