"use strict";
exports.__esModule = true;
var tslib_1 = require("tslib");
var template_processor_1 = (0, tslib_1.__importDefault)(require("./template-processor"));
var param_1 = require("../ast-parser/param");
var constants_1 = require("../ast-parser/constants");
var typedArrays = {
    Int8Array: '[object Int8Array]',
    Int16Array: '[object Int16Array]',
    Int32Array: '[object Int32Array]',
    Uint8Array: '[object Uint8Array]',
    Uint8ClampedArray: '[object Uint8ClampedArray]',
    Uint16Array: '[object Uint16Array]',
    Uint32Array: '[object Uint32Array]',
    Float32Array: '[object Float32Array]',
    Float64Array: '[object Float64Array]'
};
var TypedArrayProcessor = /** @class */ (function (_super) {
    (0, tslib_1.__extends)(TypedArrayProcessor, _super);
    function TypedArrayProcessor() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    TypedArrayProcessor.prototype.decode = function (deserializer, args, result, prop) {
        var length = deserializer.readUInt32();
        var arrayOf = args[0];
        if (arrayOf.type !== param_1.Params.Generic) {
            throw TypedArrayProcessor.errorNonGeneric;
        }
        var buffer = deserializer.readBuffer(length);
        var output;
        switch (arrayOf.name) {
            case constants_1.Generics.UInt8:
                output = buffer;
                break;
            default:
                throw TypedArrayProcessor.errorUnexpectedValue;
        }
        result[prop] = output;
    };
    TypedArrayProcessor.prototype.encode = function (serializer, args, input) {
        var arrayOf = args[0];
        if (arrayOf.type !== param_1.Params.Generic) {
            throw TypedArrayProcessor.errorNonGeneric;
        }
        this._encode(serializer, arrayOf, input);
    };
    TypedArrayProcessor.prototype._encode = function (serializer, arrayOf, input) {
        var valid = false;
        var safeName = Object.prototype.toString.call(input);
        switch (arrayOf.name) {
            case constants_1.Generics.Int8:
                valid = typedArrays.Int8Array === safeName;
                break;
            case constants_1.Generics.UInt8:
                valid = typedArrays.Uint8Array === safeName;
                break;
            case constants_1.Generics.Int16:
                valid = typedArrays.Int16Array === safeName;
                break;
            case constants_1.Generics.UInt16:
                valid = typedArrays.Uint16Array === safeName;
                break;
            case constants_1.Generics.Int32:
                valid = typedArrays.Int32Array === safeName;
                break;
            case constants_1.Generics.UInt32:
                valid = typedArrays.Uint32Array === safeName;
                break;
            default:
                valid = Buffer.isBuffer(input);
        }
        if (!valid) {
            throw TypedArrayProcessor.errorUnexpectedValue;
        }
        var length = input.byteLength;
        serializer.writeUInt32(length);
        serializer.writeBuffer(Buffer.from(input));
    };
    TypedArrayProcessor.errorNonGeneric = new Error("Input for typed array must be generic");
    TypedArrayProcessor.errorUnexpectedValue = new Error("Unexpected typed array type for typed array encoder");
    return TypedArrayProcessor;
}(template_processor_1["default"]));
exports["default"] = TypedArrayProcessor;
